import { system,ItemStack,world,EquipmentSlot,EntityComponentTypes } from "@minecraft/server";
import { RawtextBuilder, showGuidePage, showPage } from "./GuideBook";
import { ModalFormData, ActionFormData, MessageFormData } from "@minecraft/server-ui";

export const VERSION = "1.1.0";
export const VERSION_HISTORY = ["1.1.0"];

function removeSingleItem(item, player) {
  const equippable = player.getComponent("equippable");
  const mainHandItem = equippable.getEquipment(EquipmentSlot.Mainhand);

  if (mainHandItem && mainHandItem.typeId === item.typeId) {
    if ((item.amount == 1)) {
      equippable.setEquipment(
        EquipmentSlot.Mainhand,
        new ItemStack("minecraft:air")
      );
    } else {
      const newItemStack = new ItemStack(item.typeId, mainHandItem.amount - 1);
      equippable.setEquipment(EquipmentSlot.Mainhand, newItemStack);
    }
  }
}

system.runInterval(() => {
  for (const player of world.getAllPlayers()) {
    const inventory = player.getComponent(EntityComponentTypes.Inventory);

    if (inventory === undefined) return;
    const container = inventory.container;
    if (container === undefined) return;

    // Update inventory items with the typeId "sg_warp:guide"
    for (let i = 0; i < container.size; i++) {
      const itemStack = container.getItem(i);
      if (itemStack === undefined) continue;
      if (itemStack.typeId === "sg_warp:guide") {
        const lore = itemStack.getLore() || []; // Get the existing lore or start with an empty array
        if (!lore.includes("§r§7Bedrock Essential")) {
          lore.push("§r§7Bedrock Essential");
          itemStack.setLore(lore);
          container.setItem(i, itemStack);
        }
      }
    }
  }
});








world.afterEvents.playerSpawn.subscribe((event) => {
  const { initialSpawn, player } = event;
  if (!initialSpawn) return;

  const version = player.getDynamicProperty("sg_warp_version");
  if (version === undefined) {
    player.setDynamicProperty("sg_warp_version", VERSION);
    if (player.hasTag("sg_warp_received_guide")) {
      player.removeTag("sg_warp_received_guide");
      player.sendMessage({
        translate: "sg_warp.announce_update",
        with: [VERSION],
      });
      return;
    }
    player.dimension.spawnItem(
      new ItemStack("sg_warp:guide"),
      player.location
    );
  } else if (version !== VERSION) {
    player.setDynamicProperty("sg_warp_version", VERSION);
    player.sendMessage({
      translate: "sg_warp.announce_update",
      with: [VERSION],
    });
  }
});

// Define the icons and their texture paths
const ICONS = [
  { name: "Cave", path: "textures/sg/home/icons/cave.png" },
  { name: "Bell", path: "textures/sg/home/icons/bell.png" },
  { name: "Book", path: "textures/sg/home/icons/book.png" },
  { name: "Bookshelf", path: "textures/sg/home/icons/bookshelf.png" },
  { name: "Chest", path: "textures/sg/home/icons/chest.png" },
  { name: "Crafting Table", path: "textures/sg/home/icons/crafting_table.png" },
  { name: "Crown", path: "textures/sg/home/icons/crown.png" },
  { name: "Day", path: "textures/sg/home/icons/day.png" },
  { name: "Farming", path: "textures/sg/home/icons/farming.png" },
  { name: "Flowers", path: "textures/sg/home/icons/flowers.png" },
  { name: "Midnight", path: "textures/sg/home/icons/midnight.png" },
  { name: "Nature", path: "textures/sg/home/icons/nature.png" },
  { name: "Night", path: "textures/sg/home/icons/night.png" },
  { name: "Player", path: "textures/sg/home/icons/player.png" },
  { name: "Players", path: "textures/sg/home/icons/players.png" },
  { name: "Star", path: "textures/sg/home/icons/star.png" },
  { name: "Sunrise", path: "textures/sg/home/icons/sunrise.png" },
  { name: "Villager", path: "textures/sg/home/icons/villager.png" },
  { name: "World", path: "textures/sg/home/icons/world.png" },
];

function roundCoordinates(value) {
  return Math.floor(value);
}



// Function to create a new warp location
function createNewWarp(player) {
  const form = new ModalFormData();
  form.title("Create New Warp");
  
  // Populate the dropdown with icon names
  const iconNames = ICONS.map(icon => icon.name);
  form.textField("This warp will be created at your current location.\n\nWarp Name", "Enter Warp Name");
  form.dropdown("\nIcon", iconNames, 0);

  form.show(player).then((res) => {
    if (res.canceled) return;

    const name = res.formValues[0];
    const iconName = iconNames[res.formValues[1]]; // Get the selected icon name
    const icon = ICONS.find(icon => icon.name === iconName).path; // Get the corresponding texture path
    const x = roundCoordinates(player.location.x);
    const y = roundCoordinates(player.location.y);
    const z = roundCoordinates(player.location.z);

    if (name) {
      const tag = `sg_warp:${name}.${iconName}.${x}.${y}.${z}`;
      player.addTag(tag);
      player.sendMessage("§aNew warp location created.");
      showWarps(player);
    } else {
      player.sendMessage("§cPlease fill in a name.");
    }
  });
}

// Function to manage a selected warp location
function manageWarp(player, name, iconName, coordinates) {
  const iconPath = ICONS.find(icon => icon.name === iconName).path; // Get the texture path

  const form = new ActionFormData();
  form.title(`${name}`);
  form.body(`Manage your warp location:\n\n§lName:§r §e${name}\n§r§lCoordinates:§r §e${coordinates}\n§r§lIcon:§r §e${iconName}\n\n`);

  form.button("§aWarp", "textures/ui/send_icon.png");
  form.button("§cCancel", "textures/ui/cancel.png");
  form.button("§lDelete", "textures/ui/trash.png");

  form.show(player).then((res) => {
    if (res.canceled || res.selection < 0) return;

    switch (res.selection) {
      case 0: // Warp
        const [x, y, z] = coordinates.split(",");
        const adjustedX = parseFloat(x) + 0.5;
        const adjustedZ = parseFloat(z) + 0.5;
        player.teleport({ x: adjustedX, y: parseFloat(y), z: adjustedZ });
        player.sendMessage("§aWarping to location...");
        break;

      case 1: // Cancel
        player.sendMessage("§cWarping canceled.");
        break;

      case 2: // Delete
        const tagToRemove = `sg_warp:${name}.${iconName}.${coordinates.replace(/,/g, ".")}`;
        player.removeTag(tagToRemove);
        player.sendMessage("§cWarp location deleted.");
        showWarps(player)
    }
  });
}

function showWarps(player) {
  // Create and show the modal form for warp locations
  const form = new ActionFormData();
  form.title({
    translate: "sg_warp.guide.warp.title",
  });
  form.body("Select a warp location:\n §7(or create a new one)\n\n");

  // Get warp locations from player tags
  const warpLocations = [];
  const tags = player.getTags();
  tags.forEach((tag) => {
    if (tag.startsWith("sg_warp:")) {
      const [name, iconName, x, y, z] = tag.substring("sg_warp:".length).split(".");
      warpLocations.push({ name, iconName, coordinates: `${x},${y},${z}` });
    }
  });

  warpLocations.forEach((warp, index) => {
    const iconPath = ICONS.find(icon => icon.name === warp.iconName).path;
    form.button(`${warp.name}\n§o(${warp.coordinates})`, iconPath);
  });

  form.button("Create New Warp", "textures/ui/color_plus.png");

  form.show(player).then((res) => {
    if (res.canceled || res.selection < 0) return; // Ensure the selection is valid

    const selectedOption = res.selection;

    if (selectedOption === warpLocations.length) {
      // Create New Warp selected
      createNewWarp(player);
    } else {
      // Selected a warp location
      const { name, iconName, coordinates } = warpLocations[selectedOption];
      manageWarp(player, name, iconName, coordinates);
    }
  });
}




const BUTTONS = [
{
  // OPTIONS
  translation_key: "sg_warp.guide.warp.title",
  icon_path: "textures/ui/world_glyph_color.png",
  action: (player) => {
    showWarps(player)
  },
},
  {
    // CHANGELOG
    translation_key: "sg_warp.guide.changelog.title",
    icon_path: "textures/ui/book_addtextpage_default.png",
    action: (player) => {
      const body = new RawtextBuilder();
      body.pushLine({
        translate: "sg_warp.guide.changelog.body",
        with: [VERSION],
      });
      body.pushLine();
      for (const version of VERSION_HISTORY) {
        body.pushLine({
          translate: "sg_warp.guide.changelog.subtitle",
          with: [version],
        });
        body.pushLine();
        body.pushLine({
          translate: `sg_warp.guide.changelog.${version}`,
          with: ["\n"],
        });
        body.pushLine();
        body.pushLine();
      }
      showPage(player, "sg_warp.guide.changelog.title", body.build(), [
        {
          translation_key: "sg_warp.guide.common.back_button",
          action: (player) => showMainPage(player),
        },
      ]);
    },
  },
  {
    translation_key: "sg_warp.guide.common.exit_button",
  },
];

const showMainPage = (player) => {
  showGuidePage(player, "main", 4, BUTTONS);
};

world.afterEvents.itemUse.subscribe((event) => {
  system.run(() => {
    if (event.itemStack.typeId === "sg_warp:guide") {
      showMainPage(event.source);
    }
  });
});